; timer_handler.ms
;	timer interrupt handling routine
;	Michael Hope, 1998
;	michaelh@earthling.net
;	Distrubuted under the Artistic License - see www.opensource.org
;
	.include        "global.mh"
	INCLUDE		"wait_hbl.mh"

	.globl  .copy_vram
	.globl	.cr_curs
	.globl	.adv_curs
	.globl	.cury, .curx
	.globl	.display_off

	font_handle_sizeof	=	3
	font_handle_font	=	1
	font_handle_first_tile	=	0

	FONT_256ENCODING	=	0
	FONT_128ENCODING	=	1
	FONT_NOENCODING		=	2

	FONT_BCOMPRESSED	=	2
	
	FONT_BMODE_NOSCROLL	=	0
	FONT_BMODE_NOINTERP	=	1

	.CR     		= 	0x0A          ; Unix
	.MAX_FONTS		= 	6

	.module font.ms

	.area	_BSS
font_current::
	.ds	font_handle_sizeof

font_first_free_tile::
	.ds	1
.font_temp:
	.ds	1
_font_fg_colour::
font_fg_colour::
	.ds	1
_font_bg_colour::
font_bg_colour::
	.ds	1
_font_mode::
font_mode::
	.ds	1
.font_table::
	.ds	font_handle_sizeof*.MAX_FONTS
	
	.area   _CODE

	;; Copy part (size = DE) of the VRAM from (BC) to (HL)
	; Note:  Might miss the last byte.  I'm not really sure.
.copy_tiles::
	ld	a,d
	or	e
	ret	z

	ld	a,h
	cp	#0x98
	jr	c,4$
	sub	#0x98-0x88
	ld	h,a
4$:
	xor	a
	cp	e		; Special for when e=0 you will get another loop
	jr	nz,1$
	dec	d
1$:
	WAIT_HBL
	ld	a,(bc)
	ld	(hl),a

	inc	l
	jr	nz,2$
	inc	h
	ld	a,h		; Special wrap-around
	cp	#0x98
	jr	nz,2$
	ld	h,#0x88
2$:
	inc	bc
	dec	e
	jr	nz,1$
	dec	d
	ld	a,#0xff
	cp	d
	JR	NZ,1$
	RET

font_copy_special::
	ld	a,d
	or	e
	ret	z		; Do nothing

	ld	a,h
	cp	#0x98
	jr	c,font_copy_loop
	sub	#0x98-0x88
	ld	h,a
font_copy_loop:
	ld	a,(bc)
	inc	bc
	push	bc
	ld	b,a
	ld	c,a
	ld	a,(font_fg_colour)
	bit	0,a
	jr	nz,font_copy_grey1
	ld	b,#0
font_copy_grey1:
	bit	1,a
	jr	nz,font_copy_grey2
	ld	c,#0
font_copy_grey2:
	ld	a,(font_bg_colour)
	bit	0,a
	jr	z,font_copy_bg_grey1
	ld	a,#0xFF
	xor	b
	ld	b,a
	ld	a,(font_bg_colour)
font_copy_bg_grey1:
	bit	1,a
	jr	z,font_copy_bg_grey2
	ld	a,#0xFF
	xor	c
	ld	c,a
font_copy_bg_grey2:
	WAIT_HBL
	ld	(hl),b
	inc	hl
	WAIT_HBL
	ld	(hl),c
	inc	hl
	ld	a,h
	cp	#0x98
	jr	nz,1$
	ld	h,#0x88
1$:
	pop	bc
	dec	de
	dec	de
	ld	a,d
	or	e
	jr	nz,font_copy_loop
	ret
	
; Load the font HL
font_load::
	di
	call	.display_off
	push	hl

	ld	hl,#.font_table+font_handle_font
	ld	b,#.MAX_FONTS
font_load_find:
	ld	a,(hl)		; Check to see if this entry is free
	inc	hl		; Free is 0000 for the font pointer
	or	(hl)
	cp	#0
	jr	z,font_load_found

	inc	hl
	inc	hl
	dec	b
	jr	nz,font_load_find
	pop	hl
	ld	hl,#0
	jr	font_load_exit	; Couldn't load font
font_load_found:
				; HL points to the end of the free font table entry
	pop	de
	ld	(hl),d		; Copy across the font struct pointer
	dec	hl
	ld	(hl),e

	ld	a,(font_first_free_tile)
	dec	hl
	ld	(hl),a		

	push	hl
	call	font_set	; Copy font pointed to by HL to current
	
	ld	hl,#font_current+font_handle_font
	ld	a,(hl+)
	ld	h,(hl)
	ld	l,a

	inc	hl		; Points to the 'tiles required' entry
	push	hl
	ld	l,(hl)
	ld	h,#0
	add	hl,hl
	add	hl,hl
	add	hl,hl
	add	hl,hl
	ld	e,l		; DE now has the length of the tile data
	ld	d,h

	pop	hl
	dec	hl
	ld	a,(hl)
	ld	(.font_temp),a
	and	#3			; Only lower 2 bits set encoding table size

	ld	bc,#128
	cp	#FONT_128ENCODING	; 0 for 256 char encoding table, 1 for 128 char
	jr	z,font_load_copy
	ld	bc,#0
	cp	#FONT_NOENCODING
	jr	z,font_load_copy
	ld	bc,#256			; Must be 256 element encoding
font_load_copy:
	inc	hl
	inc	hl		; Points to the start of the encoding table
	add	hl,bc		; HL points to the start of the tile data

	ld	c,l
	ld	b,h

	push	bc		; Tile data
	push	de		; Length of tile data
	ld	a,(font_current+font_handle_first_tile)	; First tile used for this font
	ld	l,a
	ld	h,#0
	add	hl,hl
	add	hl,hl
	add	hl,hl
	add	hl,hl

	push	hl
	ld	a,#0x90
	add	a,h
	ld	h,a
	
	ld	a,(.font_temp)
	bit	FONT_BCOMPRESSED,a
	jr	nz,font_load_compressed
	CALL    .copy_tiles
	jr	font_load_done
font_load_compressed:
	call	font_copy_special
font_load_done:

	pop	hl
	pop	de
	pop	bc
				; Increase the 'first free tile' counter
	ld	a,(font_current+font_handle_font+0)
	ld	l,a
	ld	a,(font_current+font_handle_font+1)
	ld	h,a

	inc	hl
	ld	a,(font_first_free_tile)
	add	a,(hl)
	ld	(font_first_free_tile),a

	pop	hl		; Return font setup in HL
font_load_exit:
	;; Turn the screen on
	LDH     A,(.LCDC)
	OR      #0b10000001     ; LCD           = On
				; BG            = On
	AND     #0b11100111     ; BG Chr        = 0x8800
				; BG Bank       = 0x9800
	LDH     (.LCDC),A

	EI                      ; Enable interrupts

	RET

font_set::
	ld	a,(hl+)
	ld	(font_current),a
	ld	a,(hl+)
	ld	(font_current+1),a
	ld	a,(hl+)
	ld	(font_current+2),a
	ret
	
	;; Print a character with interpretation
.mput_char::
;	CP      #.CR
;	JR      NZ,1$
	jr	1$
	CALL    .cr_curs
	RET
1$:
	CALL    .mset_char
	CALL    .adv_curs
	RET

	;; Print the character in A
.mset_char:
	PUSH    BC
	PUSH    DE
	PUSH    HL
				; Compute which tile maps to this character
	ld	e,a
	ld	hl,#font_current+font_handle_font
	ld	a,(hl+)
	ld	h,(hl)
	ld	l,a
	ld	a,(hl+)
	and	#3
	cp	#FONT_NOENCODING
	jr	z,mset_char_no_encoding
	inc	hl
				; Now at the base of the encoding table
				; E is set above
	ld	d,#0
	add	hl,de
	ld	e,(hl)		; That's the tile!
mset_char_no_encoding:
	ld	a,(font_current+0)
	add	a,e
	ld	e,a

	LD      A,(.cury)       ; Y coordinate
	LD      L,A
	LD      H,#0x00
	ADD     HL,HL
	ADD     HL,HL
	ADD     HL,HL
	ADD     HL,HL
	ADD     HL,HL
	LD      A,(.curx)       ; X coordinate
	LD      C,A
	LD      B,#0x00
	ADD     HL,BC
	LD      BC,#0x9800
	ADD     HL,BC
1$:
	LDH     A,(.STAT)
	AND     #0x02
	JR      NZ,1$
	LD      (HL),E
	POP     HL
	POP     DE
	POP     BC
	RET

_mput_char::
	LDA     HL,2(SP)        ; Skip return address
	LD      A,(HL)          ; A = c
	CALL    .mput_char
	RET

_mset_char::
	push	bc
	LDA     HL,4(SP)        ; Skip return address
	LD      A,(HL)          ; A = c
	CALL    .mset_char
	pop	bc
	RET

_font_load::
	push	bc
	LDA     HL,4(SP)        ; Skip return address and bc
	LD      A,(HL)          ; A = c
	inc	hl
	ld	h,(hl)
	ld	l,a
	call    font_load
	push	hl
	pop	de		; Return in DE
	pop	bc
	RET

_mprint_string::
	push	bc
	LDA     HL,4(SP)        ; Skip return address
	LD      A,(HL)          ; A = c
	inc	hl
	ld	h,(hl)
	ld	l,a
	CALL    .mprint_string
	pop	bc
	RET

_font_set::
	push	bc
	LDA     HL,4(SP)        ; Skip return address
	LD      A,(HL)          ; A = c
	inc	hl
	ld	h,(hl)
	ld	l,a
	call	font_set
	pop	bc
	ld	de,#0		; Always good...
	RET


_font_init::
	push	bc
	ld	hl,#.font_table
	ld	b,#font_handle_sizeof*.MAX_FONTS
	xor	a
	ld	(font_first_free_tile),a
1$:
	ld	(hl+),a
	dec	b
	jr	nz,1$
	ld	a,#3
	ld	(font_fg_colour),a
	ld	a,#0
	ld	(font_bg_colour),a

	xor	a
	ld	(font_mode),a
	
	pop	bc
	ret
	
.mprint_string::
	ld	a,(hl)
	cp	#0
	ret	z
	push	hl
	call	.mput_char
	pop	hl
	inc	hl
	jr	.mprint_string

_print_repeat::
	push	bc
	lda	hl,4(sp)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	xor	a
	cp	d
	jr	nz,_print_repeat_go
	pop	bc
	ret
_print_repeat_go:
	ld	a,e
	push	de
	call	.mput_char
	pop	de
	dec	d
	jr	nz,_print_repeat_go
	pop	bc
	ret

_mgotoxy::
	lda	hl,2(sp)
	ld	a,(hl+)
	ld	(.curx),a
	ld	a,(hl)
	ld	(.cury),a
	ret
	
