/* metro.c
	Metronome demo for the 'ui Simple user interface for the GameBoy'
	Michael Hope, 1998
	michaelh@earthling.net
	Distrubuted under the Artistic License - see www.opensource.org
*/
#include "ui.h"
#include "font.h"
#include "timer.h"
#include <string.h>

/* Fonts to use */
extern UBYTE font_draw[];
extern UBYTE font_spect[];

/* Event codes for the buttons */
#define ID_BUT_ABOUT	1
#define ID_BUT_FLASH	2
#define ID_BUT_ABOUT_OK	3

mscreen screen_main;		/* Cant init now as we need to be able
				   to modify the focus field */
/* Defined as extern as the parent/children object fields cause a recursive
   link otherwise */
extern mwindow win_main;
extern mframe frame_rate, frame_beats, frame_accent;

/* Image for about window */
extern UBYTE bjork[];

/* Objects that need to be in RAM (modifiable).  Inited by
   there corresponding *_init object
*/
mslider slider_rate, slider_beats, slider_accent;
mimage	img_about;

/* Hooks for the various sliders.  These are called wheneve
   a slider changes */
void rate_change(pmobject This);
void beats_change(pmobject This);
void accent_change(pmobject This);

/* A slider, static and frame for the second accent bar */
mstatic static_accent = {
	&static_handler, NULL, (pmobject)&frame_accent, NULL,
	NULL,
	0, 0, 13, 1,	/* With respect to the frame */
	"Second accent"
};
	
mslider slider_accent_init = {
	&slider_handler, NULL, (pmobject)&frame_accent, NULL,
	(pmobject)&static_accent,
	0, 1, 18, 1,
	2, 2, 8, 1,	/* Start at 2, min 2, max 8, steps of 1 */
	3,		/* Leave 3 spaces for the value */
	accent_change
};

mframe	frame_accent = {
	&frame_handler, (pmobject)&slider_accent, (pmobject)&screen_main, NULL,
	NULL,
	1, 2+3+3, 18, 2
};

/* A slider, static and frame for the beats bar */
mstatic static_beats = {
	&static_handler, NULL, (pmobject)&frame_beats, NULL,
	NULL,
	0, 0, 9, 1,
	"Beats/Bar"
};
	
mslider slider_beats_init = {
	&slider_handler, NULL, (pmobject)&frame_beats, NULL,
	(pmobject)&static_beats,
	0, 1, 18, 1,
	4, 2, 8, 1,	/* Start at 4, min 2, max 8, step of 1 */
	3,
	beats_change
};

mframe	frame_beats = {
	&frame_handler, (pmobject)&slider_beats, (pmobject)&screen_main, NULL,
	(pmobject)&frame_accent, 
	1, 2+3, 18, 2
};

/* A slider, static and frame for the rate bar */
mstatic static_rate = {
	&static_handler, NULL, (pmobject)&frame_rate, NULL,
	NULL,
	0, 0, 4, 1,
	"Rate"
};
	
mslider slider_rate_init = {
	&slider_handler, NULL, (pmobject)&frame_rate, NULL,
	(pmobject)&static_rate,
	0, 1, 18, 1,
	80, 30, 150, 2,		/* Start at 80, min 30, max 150, step 2 */
	3,
	rate_change
};

mframe	frame_rate = {
	&frame_handler, (pmobject)&slider_rate, (pmobject)&screen_main, NULL,
	(pmobject)&frame_beats,
	1, 2, 18, 2
};

/* Button on the main screen labeled 'Flash' */
mbutton but_flash = {
	&button_handler, NULL, (pmobject)&screen_main, NULL,
	(pmobject)&frame_rate,
	20-9, 18-3-3, 9, 3,
	"Flash",
	ID_BUT_FLASH		/* Return ID when it's clicked upon */
};

/* Button on the main screen labeled 'About' */
mbutton but_about = {
	&button_handler,
	NULL,			/* No children */
	(pmobject)&screen_main,	/* Parent */
	NULL,			/* Which object takes focus */
	(pmobject)&but_flash,	/* Next in children */
	20-9, 18-3, 9, 3,
	"About",
	ID_BUT_ABOUT
};

/* 'About window' */
extern mwindow win_about;

/* Image made up of 16 (4*4) tiles */
mimage	img_about_init = {
	&image_handler, NULL, (pmobject)&win_about, NULL,
	NULL,
	1, 2, 4, 4,		/* 32x32 pixel = 4x4 cell image */
	NULL			/* Inited in the main loop */
};

/* Static text in About window */
mstatic stat_about_date = {
	&static_handler, NULL, (pmobject)&win_about, NULL,
	(pmobject)&img_about,
	0, 5, 15, 1, "1998", JUST_RIGHT		/* Right justified */
};

mstatic stat_about_author = {
	&static_handler, NULL, (pmobject)&win_about, NULL,
	(pmobject)&stat_about_date,
	0, 4, 15, 1, "By M.Hope", JUST_RIGHT
};

mstatic stat_about_title = {
	&static_handler, NULL, (pmobject)&win_about, NULL,
	(pmobject)&stat_about_author,
	0, 2, 15, 1,
	"Metronome",
	JUST_RIGHT
};

/* OK button in About window */
mbutton but_about_ok = {
	&button_handler, NULL, (pmobject)&win_about, NULL,
	(pmobject)&stat_about_title,
	15-6, 11-3, 6, 3,
	"OK",
	ID_BUT_ABOUT_OK
};
	
/* About window, displayed by pressing 'About' */
mwindow win_about = {
	&window_handler,
	(pmobject)&but_about_ok,	/* Children begin with the about button */
	(pmobject)&screen_main,
	NULL,				/* Lowest level window - cant pass focus on */
	NULL,				/* Last in the screens children */
	2, 2, 16, 12,
	"About",
};

int main(void)
{
	int a, i;
	/* Initalise the font system */
	font_init();
	/* Initalise the ui system, using the spectrum font for text */
	ui_init(font_draw, font_spect);
	/* Init the metronome part */
	timer_init();

	/* Init the main screen */
	screen_init(&screen_main, 0, 0, 20, 18, "Metronome");
	/* Make the main screen the focus - buttons on it are active */
	screen_main.focus = NULL;
	/* First in the children objects */
	screen_main.children = (pmobject)&but_about;
	
	/* Copy across any objects that need to be modifiable into ram */
	memcpy( &slider_rate, &slider_rate_init, sizeof(mslider));
	memcpy( &slider_beats, &slider_beats_init, sizeof(mslider));
	memcpy( &slider_accent, &slider_accent_init, sizeof(mslider));
	memcpy( &img_about, &img_about_init, sizeof(mimage));

	/* Load the font with the image in it */
	img_about.hfont = font_load(bjork);

	/* Draw the screen */
	screen_main.handler->paint((pmobject)&screen_main, 0, 0);
	while (1) {
		/* Move the pointer until A is pressed */
		pointer_loop(J_A,1);
		/* Find the ID of the clicked object */
		a = screen_main.handler->but_down((pmobject)&screen_main, 0, 0, 
			point_x>>3, point_y>>3, point_x, point_y);
		if (a!=0) {
			/* An object responded to the click */ 
			switch (a) {
				case ID_BUT_FLASH:
					/* Enter screen flash mode */
					HIDE_SPRITES;
					erase_rect((pmobject)&screen_main, 0);
					timer_flash();
					erase_rect((pmobject)&screen_main, 0);
					SHOW_SPRITES;
					screen_main.handler->paint((pmobject)&screen_main, 0, 0);
					break;
				case ID_BUT_ABOUT:
					/* Pop the About window up over the main screen */
					/* Set the focus of the screen to the window 
					   All clicks are passed directly to the window
					   so that all sliders and buttons on the screen
					   cant be clicked upon */
					screen_main.focus = (pmobject)&win_about;
					/* Redraw */
					screen_main.handler->paint((pmobject)&screen_main, 0, 0);
					break;
				case ID_BUT_ABOUT_OK:
					/* Occurs when About window is up.  Use to close About win */
					/* Take the focus back */
					screen_main.focus = NULL;
					/* Erase where the window was */
					erase_rect((pmobject)&win_about, 0);
					/* Redraw */
					screen_main.handler->paint((pmobject)&screen_main, 0, 0);
					break;					
				default:
					break;
			}
		}
	}
	return 0;
}
