/* timer.c
	Metronome timer handlers - badly named :)
	Michael Hope, 1998
	michaelh@earthling.net
	Distrubuted under the Artistic License - see www.opensource.org
*/
#include "timer.h"
#include "font.h"
#include "ui.h"

/* Tone:
	freq = 133333 / (2048-x)
	2048 - x = 133333 / freq
	x = 2048 - 133333 / freq
*/
#define TONE_NORM	1949		// E
#define TONE_HI		1964		// G
#define TONE_LOW	1923		// C

extern mslider slider_accent;
extern UWORD *beat_current_tone;
extern UWORD beat_tones[];

UBYTE accent_on;

typedef struct srect mrect;
typedef struct srect *pmrect;

struct srect {
	UBYTE left, top;
	UBYTE width, height;
};

void set_tones(void)
{
	UBYTE i;

	for (i = 0; i!=8; i++) {
		if (i==(accent_on-2))
			beat_tones[i] = TONE_LOW;
		else
			beat_tones[i] = TONE_NORM;
	}
	beat_tones[beat_last-1] = TONE_HI;
}

void accent_change(pmobject This)
{
	accent_on = ((pmslider)This)->current;
	set_tones();
}

void rate_change(pmobject This)
{
	beat_period = (UBYTE)((UWORD)TIMER_RATE*60/((pmslider)This)->current);
}

void beats_change(pmobject This)
{
	beat_current = 0;
	beat_last = ((pmslider)This)->current;
	set_tones();
}

int timer_init(void)
{
	NR52_REG = 0x80;		/* All sound on */

	beat_last = 4;
	accent_on = 0;
	beat_current = 0;
	beat_period = TIMER_RATE;
	beat_current_tone = beat_tones;

	NR12_REG = 0xF0;		/* Max volume */
	NR22_REG = 0xF0;		/* Max volume */
	NR50_REG = 0xFF;		/* Masters to MAX */
	NR51_REG = 0x33U;		/* To both channels */
					/* Sound is turned on in timer_handler */
	set_tones();
	disable_interrupts();
	add_TIM(timer_handler);
	TAC_REG = 4;				/* 4096 */
	TMA_REG = (256-(4096/TIMER_RATE));	/* /41 = 100Hz */
	set_interrupts(VBL_IFLAG | TIM_IFLAG);
	enable_interrupts();
	return 0;
}

void rect_draw(pmrect rect, UBYTE cell)
{
	UBYTE height;

	height = rect->height;
	while (height--) {
		mgotoxy(rect->left, rect->top+height-1);
		print_repeat(cell, rect->width);
	}
}

int flash_draw(UBYTE num, UBYTE of, UBYTE cell)
{
	mrect rect;
	int width, height, step, x, y, i;
	width = 20;
	height = 18;
	x = 0;
	y = 0;
	switch (of) {
		case 2:
			width/=2;
			break;
		case 3:
		case 4:
			width/=2;
			height/=2;
			break;
		case 5:
		case 6:
			width/=3;
			height/=2;
			break;
		case 7:
		case 8:
		case 9:
			width/=4;
			height/=2;
			break;
	}

	step = width;
	for (i=0; i<num; i++) {
		x+=step;
		if ((x+width)>20) {
			step = -step;
			x+=step;
			y+=height;
		}
		if (x<0) {
			step = -step;
			y+=height;
			x+=step;
		}
	}
	rect.top = y;
	rect.left = x;
	rect.width = width;
	rect.height = height;

	rect_draw(&rect, cell);
	return 0;
}

	
int timer_flash(void)
{
	UBYTE beat;

	beat = beat_current;
	font_set(hfont_draw);

	while (!(joypad()&J_A)) {
		if (beat!=beat_current) {
			flash_draw( beat, beat_last, TCLEAR);
			beat = beat_current;
			if (beat==0) 
				flash_draw( beat, beat_last, TGREY66);
			else
				if ((beat+1)==slider_accent.current)
					flash_draw( beat, beat_last, TGREY33);
				else
					flash_draw( beat, beat_last, TSOLID);
		}
		wait_vbl_done();
	}
	waitpadup();
	return 0;
}
